from awxclient import base
from awxclient import connection
from awxclient import exception

API_VER = 'v2'

class Client(object):

    def __init__(self, awx_host, username=None, password=None, token=None, api_ver=None, verify=False):
        base_url = awx_host
        if not awx_host.startswith("http"):
            base_url = f"https://{awx_host}"
        # default to 'v2' ('.../api/v2/...')
        ver = api_ver or 'v2'
        base_url = f"{base_url}/api/{ver}" 
        self._conn = connection.Connection(base_url, verify=verify)
        if username and password:
            self._conn.login(username=username, password=password)
        elif token:
            self._conn.login(token=token)

    def login(self, username=None, password=None, token=None):
        self._conn.login(username=username, password=password, token=token)

    def logout(self):
        self._conn.logout()

    @property
    def applications(self):
        return ApplicationManager(self._conn)

    @property
    def organizations(self):
        return OrganizationManager(self._conn)

    @property
    def projects(self):
        return ProjectManager(self._conn)

    @property
    def teams(self):
        return TeamManager(self._conn)

    @property
    def job_templates(self):
        return JobTemplateManager(self._conn)

    @property
    def jobs(self):
        return JobManager(self._conn)

    @property
    def job_events(self):
        return JobEventManager(self._conn)


class ApplicationManager(base.BaseManager):
    def __init__(self, conn):
        super(ApplicationManager, self).__init__(conn, 'applications')


class OrganizationManager(base.BaseManager):
    def __init__(self, conn):
        super(OrganizationManager, self).__init__(conn, 'organizations')


class ProjectManager(base.BaseManager):
    def __init__(self, conn):
        super(ProjectManager, self).__init__(conn, 'projects')


class TeamManager(base.BaseManager):
    def __init__(self, conn):
        super(TeamManager, self).__init__(conn, 'teams')


class JobTemplateManager(base.BaseManager):
    def __init__(self, conn):
        super(JobTemplateManager, self).__init__(conn, 'job_templates')

    def launch_job(self, job_template_id, data=None):
        post_data = {
            "extra_vars": data or {}
        }
        return self._conn.post(f"{self._relative_url_base}/{job_template_id}/launch/", data=post_data)


class JobManager(base.BaseManager):
    def __init__(self, conn):
        super(JobManager, self).__init__(conn, 'jobs')

    def is_job_cancellable(self, job_id):
        return self._conn.get(f"{self._relative_url_base}/{job_id}/cancel/")

    def cancel_job(self, job_id):
        return self._conn.post(f"{self._relative_url_base}/{job_id}/cancel/")

    def get_job_events(self, job_id):
        return self._conn.get(f"{self._relative_url_base}/{job_id}/job_events/")

    def get_job_labels(self, job_id):
        return self._conn.get(f"{self._relative_url_base}/{job_id}/labels/")

    def get_job_notifications(self, job_id):
        return self._conn.get(f"{self._relative_url_base}/{job_id}/notifications/")

    def relaunch_job(self, job_id):
        return self._conn.post(f"{self._relative_url_base}/{job_id}/relaunch/")

    def get_job_stdout(self, job_id):
        return self._conn.get(f"{self._relative_url_base}/{job_id}/notifications/")


class JobEventManager(base.BaseManager):
    def __init__(self, conn):
        super(JobEventManager, self).__init__(conn, 'job_events')



import requests

from awxclient import exception


class ConnectionException(exception.AWXClientRequestException):
    pass


class Token_Auth(requests.auth.AuthBase):
    def __init__(self, token):
        self.token = token

    def __call__(self, request):
        request.headers['Authorization'] = 'Bearer {0.token}'.format(self)
        return request


MAX_CONNECT_ATTEMPTS = 3

class Connection(object):

    def __init__(self, base_url, verify=False):
        self.base_url = base_url 
        self.verify = verify
        if not self.verify:
            requests.packages.urllib3.disable_warnings()
        self.session = requests.Session()

    def login(self, username=None, password=None, token=None, **kwargs):
        if username and password:
            self.session.auth = (username, password)
        elif token:
            self.session.auth = Token_Auth(token)
        else:
            self.session.auth = None

    def logout(self):
        self.session.auth = None

    def request(self, relative_endpoint, method='get', json=None, data=None, query_params=None, headers=None):
        session_request_method = getattr(self.session, method, None)
        if not session_request_method:
            raise ConnectionException(message=f"Unknown request method: {method}")

        use_endpoint = relative_endpoint
        if self.base_url.endswith('/'):
            self.base_url = self.base_url[:-1]
        if use_endpoint.startswith('/'):
            use_endpoint = use_endpoint[1:]
        url = '/'.join([self.base_url, use_endpoint])

        kwargs = dict(verify=self.verify, params=query_params, json=json, data=data)

        if headers is not None:
            kwargs['headers'] = headers
        if method in ('post', 'put', 'patch', 'delete'):
            kwargs.setdefault('headers', {})['X-CSRFToken'] = self.session.cookies.get('csrftoken')
            kwargs['headers']['Referer'] = url

        for attempt in range(0, MAX_CONNECT_ATTEMPTS):
            try:
                response = session_request_method(url, **kwargs)
                break
            except requests.exceptions.ConnectionError as err:
                if attempt == MAX_CONNECT_ATTEMPTS:
                    raise err

        try:
            response.raise_for_status()
            return response.json()
        except requests.exceptions.HTTPError:
            try:
                data = response.json()
            except ValueError:
                data = response.text
            raise exception.AWXClientRequestException(
                status=response.status_code, message=data)
        except ValueError:
            return response.text

    def delete(self, relative_endpoint):
        return self.request(relative_endpoint, method='delete')

    def get(self, relative_endpoint, query_params=None, headers=None):
        return self.request(relative_endpoint, method='get', query_params=query_params, headers=headers)

    def head(self, relative_endpoint):
        return self.request(relative_endpoint, method='head')

    def options(self, relative_endpoint):
        return self.request(relative_endpoint, method='options')

    def patch(self, relative_endpoint, json=None, data=None):
        return self.request(relative_endpoint, method='patch', json=json, data=data)

    def post(self, relative_endpoint, json=None, data=None, headers=None):
        return self.request(relative_endpoint, method='post', json=json, data=data, headers=headers)

    def put(self, relative_endpoint, json=None, data=None, headers=None):
        return self.request(relative_endpoint, method='put', json=json, data=data, headers=headers)



import json

class AWXClientRequestException(Exception):
    def __init__(self, status='', message=''):
        if isinstance(status, Exception):
            self.status = ''
            return super(AWXClientRequestException, self).__init__(*status)
        self.status = status
        self.msg = json.dumps(message)

    def __getitem__(self, val):
        return (self.status, self.msg)[val]

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return '{} - {}'.format(self.status, self.msg)



import json

from awxclient import exception

class BaseManager(object):
    def __init__(self, conn, rel_url_base):
        self._conn = conn
        self._relative_url_base = rel_url_base

    def get_all(self, **kwargs):
        # example kwargs could be {'page':2}
        return self._conn.get(self._relative_url_base, query_params=kwargs)

    def _get_id_url(self, object_id):
        return f"{self._relative_url_base}/{object_id}"

    def get_by_id(self, object_id):
        id_url = self._get_id_url(object_id)
        resource = self._conn.get(id_url)
        return resource

    def get_by_name(self, name):
        params = {"name": name}
        result = self._conn.get(self._relative_url_base, query_params=params)
        if result and result.get('count'):
            return result['results'][0]
        raise exception.AWXClientRequestException(status=404, message={"detail": "Not found."})

    def search(self, search_term):
        params = {"search": search_term}
        return self._conn.get(self._relative_url_base, query_params=params)

    def search_by_field(self, field_name, field_val):
        params = {f"{field_name}__search": field_val}
        return self._conn.get(self._relative_url_base, query_params=params)

    def create(self, json=None, data=None):
        return self._conn.post(self._relative_url_base, json=json, data=data)

    def update(self, object_id, json=None, data=None):
        return self._conn.put(self._get_id_url(object_id), json=json, data=data)

    def delete(self, object_id):
        return self._conn.delete(self._get_id_url(object_id))
